
#import "WoodOOEFilter.h"
#import "OOE_Protocols.h"
#import "TreeView.h"

static NXAtom WoodPboardType;
static NXAtom WoodOOEApplicationInfoPboardType;
static NXAtom WoodName;

@implementation WoodOOEFilter

+ initialize
{
	if([self class] == [WoodOOEFilter class]){
		WoodPboardType = NXUniqueString("Wood Pasteboard Type");
		WoodOOEApplicationInfoPboardType = NXUniqueString(OOE_APPINFO_PASTEBOARDTYPE);
		WoodName = NXUniqueString("Wood.app");
	}
	return self;
}

- (void)initListener
{
	const char *name = [self appName];
	
	appListener = [[Listener alloc] init];
	if([appListener checkInAs: name] != 0)
		NXLogError("ERROR: Could not check in MACH port %s!", name);
	[appListener setDelegate:self];
	[appListener setServicesDelegate: self];
	[appListener addPort];
}

- runAsUNIXFilter
{
	if(NXArgv[1] != NULL && strcmp(NXArgv[1], "-file") == 0){	
		strcpy(theFile, NXArgv[2]);
		thePasteboard = [Pasteboard new];		
		[self convertFileToOOE];		
		return self;
	}
	return nil;
}

- run
{
    if([self runAsUNIXFilter] != nil)
		return self;
	[self initListener];
	[Listener run];	
	return self;
}

- openTree
{
	NXTypedStream *ts;
	
	theZone = NXCreateZone(vm_page_size, vm_page_size, NO);
    NXNameZone(theZone, "Scrapper");
	ts = NXOpenTypedStreamForFile(theFile,NX_READONLY);
	NXSetTypedStreamZone(ts, theZone);
    theTree = NXReadObject(ts);
	NXCloseTypedStream(ts);
	return self;
}

- closeTree
{
	[theTree free];
	NXDestroyZone(theZone);
	return self;
}

- convertFileToOOE
{
	const char *types[3];
	int k = 0;
	
	[self openTree];
	types[k++] = NXPostScriptPboardType;
	types[k++] = WoodPboardType;
	types[k++] = WoodOOEApplicationInfoPboardType;
	[thePasteboard declareTypes:types num:k owner:NXApp];		
	[self writeEPS];
	[self writeTree];
	[self writeOAI];
	[self closeTree];
	return self;
}

- convertFileToEPS
{
	const char *types[3];
	int k = 0;
	
	[self openTree];
	types[k++] = NXPostScriptPboardType;
	[thePasteboard declareTypes:types num:k owner:NXApp];
	[self writeEPS];
	[self closeTree];
	return self;
}

- writeEPS
{
    Window *w;
    NXStream *stream;
    TreeView *scrapper;
    NXRect scrapperFrame = {{0.0, 0.0}, {11.0*72.0, 14.0*72.0}};

    scrapper = [[TreeView allocFromZone:theZone] initFrame:&scrapperFrame];
	[scrapper checkResize:NULL for:theTree];
	[scrapper getFrame:&scrapperFrame];
    w = [[Window allocFromZone:theZone] initContent:&scrapperFrame
					   style:NX_PLAINSTYLE
					 backing:NX_NONRETAINED
				      buttonMask:0
					   defer:NO];
    [w reenableDisplay];
    [w setContentView:scrapper];
    stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
    [scrapper writePSToStream:stream usingTree:theTree];
	NXSeek(stream, 0, NX_FROMSTART);
	NX_DURING
    	[thePasteboard writeType:NXPostScriptPboardType fromStream:stream];
	NX_HANDLER
		NXRunAlertPanel(0,"NX_pasteboardComm Exception",0,0,0);
	NX_ENDHANDLER
    NXCloseMemory(stream, NX_FREEBUFFER);
    [w free]; 
    return self;
}

- writeTree
{
	NXStream *stream;
	NXTypedStream *typedStream;
	
   	stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	typedStream = NXOpenTypedStream(stream, NX_WRITEONLY);
	[theTree writeToPBStream:typedStream];
	NXSeek(stream, 0, NX_FROMSTART);
	NX_DURING
    	[thePasteboard writeType:WoodPboardType fromStream:stream];
	NX_HANDLER
		NXRunAlertPanel(0,"NX_pasteboardComm Exception",0,0,0);
	NX_ENDHANDLER
	NXCloseTypedStream(typedStream);
	NXCloseMemory(stream, NX_FREEBUFFER);
	return self;
}

- writeOAI
{
	[thePasteboard writeType:WoodOOEApplicationInfoPboardType data:WoodName length:strlen(WoodName)];
	return self;
}

- (int)wood2ooe:aPasteboard
    userData:(const char *)userData
    error:(char **)msg
{
	char *data = NULL; 
	int length = 0;
	const char *filetype = "NXTypedFilenamePboardType:wood";
	const char *type = NULL;
	
	type = [aPasteboard findAvailableTypeFrom:&filetype num:1];
	if(type == NULL){
		*msg = "No NXTypedFilenamePboardType:wood in pboard";
		NXLogError(*msg);
		return 1;
	}
	[aPasteboard readType:filetype data:&data length:&length];
	strcpy(theFile, data);
	thePasteboard = aPasteboard;
	[self convertFileToOOE];
	[thePasteboard deallocatePasteboardData:data length:length];
	return 0;
}

- (int)wood2eps:aPasteboard
    userData:(const char *)userData
    error:(char **)msg
{
	char *data = NULL; 
	int length = 0;
	const char *filetype = "NXTypedFilenamePboardType:wood";
	const char *type = NULL;
	
	type = [aPasteboard findAvailableTypeFrom:&filetype num:1];
	if(type == NULL){
		*msg = "No NXTypedFilenamePboardType:wood in pboard";
		NXLogError(*msg);
		return 1;
	}
	[aPasteboard readType:filetype data:&data length:&length];
	strcpy(theFile, data);
	thePasteboard = aPasteboard;
	[self convertFileToEPS];
	[thePasteboard deallocatePasteboardData:data length:length];
	return 0;
}

@end


