/*
 *  Copyright (C) 1993  Robert Davis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of Version 2, or any later version, of 
 *  the GNU General Public License as published by the Free Software 
 *  Foundation.
 */

/* $Id: GnuplotPlot.h,v 1.8 1993/05/18 03:55:11 davis Exp $ */

#import <objc/Object.h>
#import <appkit/Pasteboard.h>		/* NXAtom			*/
#import <dpsclient/event.h>		/* NXSize			*/

@class Gnuplot, Status;

#define UPDATE_NEVER	0		/* Constant update types	*/
#define	UPDATE_ALWAYS	1
#define	UPDATE_NOT3D	2

@interface GnuplotPlot:Object
{
    Gnuplot	*controller;		/* NXApp's delegate		*/
    Status	*status;		/* gnuplot Status		*/
    id		plotScrollView;		/* CellScrollView		*/
    id		plotView;		/* PlotView			*/

    id		window;			/* Our window			*/
    id		stringSet;		/* controller's string set	*/

    NXZone	*zone;			/* Our zone			*/

    char	*fullPath;		/* "UntitledXX" if untitled	*/
    BOOL	isTitled;
    char	*readText;		/* Text read from a file	*/
}

+ initialize;
+ setConstantUpdate:(int)updateType;
+ setHalvePlot:(BOOL)condition;

- init;
- initFromFile:(const char *) filename;	/* Filename is full path	*/
- free;

/*** Target/Action ***/
- save:sender;
- saveAs:sender;
- saveTo:sender;
- revertToSaved:sender;
- close:sender;
- print:sender;



/*** Setting Instance Variables ***/
- setName: (const char *) name;
- (const char *) name;

- setCurrentFont:aFont;		/* Awkward for this class to handle this */
- currentFont;

- window;
- (Status *)status;		/* The gnuplot Status object		*/



/*** Plotting ***/
- plotFromFile:(const char *)aFullPath;
- plot:sender;
- reportScriptError: sender;

- addDataFile:(const char *)aPath;



/*** Document ***/
- setDocEdited:(BOOL) state;
- (BOOL) isDocEdited;
- hideDocument:sender;



/*** Services ***/
- validRequestorForSendType:(NXAtom)sendType andReturnType:(NXAtom)returnType;
- (BOOL)writeSelectionToPasteboard:pboard types:(NXAtom *)types;



/*** Autoupdate methods ***/
- (BOOL)validateCommand:menuCell;



/*** Window Delegate Methods ***/
- windowDidBecomeMain:sender;
- windowDidMiniaturize:sender;
- windowDidResignMain:sender;
- windowDidResize:sender;
- windowWillClose:sender;
- windowWillResize:sender toSize:(NXSize *) frameSize;



/*** Status Delegate Methods ***/
- settingsDidChange:sender;		



@end
