/* File: Magnify.h - Application class for 'Magnify'
 *
 * By: Christopher Lane
 * Symbolic Systems Resources Group
 * Knowledge Systems Laboratory
 * Stanford University
 *
 * Date: 9 January 1992
 *
 * Copyright: 1990, 1991 & 1992 by The Leland Stanford Junior University.
 * This program may be distributed without restriction for non-commercial use.
 */
 
#import <appkit/appkit.h>

#import "MagnifyView.h"

#define VERSION __DATE__
#define APPLICATION "Magnify"

#define SCALEDEFAULTSTRING "Scale"
#define DEFAULTSCALESTRING "4.0"
#define DEFAULTSCALE ((float) 4.0)

#define GRIDDEFAULTSTRING "Grid"
#define DEFAULTGRIDSTRING "-1.0"
#define DEFAULTGRID ((float) -1.0) /* aka 'turned off' */

#define CURSORDEFAULTSTRING "Cursor"
#define DEFAULTCURSORSTRING "No"

#define UPDATEDEFAULTSTRING "Update"
#define DEFAULTUPDATESTRING "-1.0"
#define DEFAULTUPDATE ((double) -1.0) /* aka 'turned off' */

#define FROZENMENUCELLTAG (1)
#define FROZENMENUTITLE "Freeze"
#define UNFROZENMENUTITLE "Unfreeze"

#define OFFSTRING "Off"
#define YESSTRING "Yes"

#define HALF ((float) 0.5)

#define FLOAT "%f"
#define DOUBLE "%F"

#define getDefault(s) NXGetDefaultValue(APPLICATION, s)
#define registerDefaults(s) NXRegisterDefaults(APPLICATION, s)

@interface Magnify : Application
{ 
	MagnifyView *view;
	DPSTimedEntry timedEntry;
	id scaleField, scaleSlider;
	id gridField, gridSlider;
	id updateField, updateSlider;
	id cursorButton;
	id version;
}

+ new;

- appDidInit:sender;
- hide:sender;
- unhide:sender;
- free;

- changeScale:sender;
- showScale:(float) scale andSlider:(BOOL) flag;

- changeGrid:sender;
- showGrid:(float) grid andSlider:(BOOL) flag;

- changeUpdate:sender;
- showUpdate:(double) update andSlider:(BOOL) flag;

- (MagnifyView *) getView;
- setView:(MagnifyView *) anObject;

- setVersion:anObject;
- (int) msgVersion:(const char **) aString ok:(int *) flag;

@end
